#include "testlib.h"
#include <array>
#include <vector>
#include <algorithm>
 
using namespace std;

/* 
groups: 
0: samples                n <=    5   q <=    5
1: at most two bamboos    n <=  3e5   q <=  3e5
2: any number of bamboos  n <=  3e5   q <=  3e5
3: any slow solution      n <=   50   q <=   50
4: n^2 log n              n <= 1000   q <= 1000
5: nq                     n <=  3e5   q <=  100
6: full binary tree (nhd) n <=  3e5   q <=  3e5
7: nh                     n <=  3e5   q <=  3e5   h <= 100
8: min heap               n <=  3e5   q <=  3e5
9: full                   n <=  3e5   q <=  3e5
*/
 
const int kMaxNQ = 300'000;
const auto kMaxN = to_array({5, kMaxNQ, kMaxNQ, 50, 1000, kMaxNQ, kMaxNQ, kMaxNQ, kMaxNQ, kMaxNQ}); 
const auto kMaxQ = to_array({5, kMaxNQ, kMaxNQ, 50, 1000, 100, kMaxNQ, kMaxNQ, kMaxNQ, kMaxNQ});
 
int main(int argc, char **argv) {
  registerValidation(argc, argv);
  int group = stoi(validator.group());

  int n = inf.readInt(2, kMaxN[group], "n"); 
  inf.readSpace();

  int q = inf.readInt(1, min(n, kMaxQ[group]), "q");
  inf.readEoln();

  vector<int> parents(n);
  for (int i = 1; i < n; ++i) {
    parents[i] = inf.readInt(1, i, "p[" + vtos(i + 1) + "]");
    if (i + 1 < n) {
      inf.readSpace();
    }
  }
  inf.readEoln();

  auto ensure_distinct = [](vector<int> values) {
    sort(values.begin(), values.end());
    ensure(unique(values.begin(), values.end()) == values.end());
  };

  vector<int> skills = inf.readInts(n, 1, n, "s");
  ensure_distinct(skills);
  inf.readEoln();

  vector<int> queries = inf.readInts(q, 1, n, "v");
  ensure_distinct(queries);
  inf.readEoln();
  inf.readEof();

  for (int i = 0; i < n; ++i) {
    parents[i] -= 1;
    skills[i] -= 1; 
  }
  
  for (int i = 0; i < q; ++i) {
    queries[i] -= 1;
  }

  if (group == 1 || group == 2) { // bamboos
    int max_bamboos = (group == 1 ? 2 : n - 1); 
    int bamboos_count = 0;
    for (int i = 1; i < n; ++i) {
      ensuref((parents[i] == 0) || (parents[i] == i - 1), "tree is not bamboo-like");
      if (parents[i] == 0) {
        ++bamboos_count; 
      }
    }
    ensure(bamboos_count <= max_bamboos);
  } 

  if (group == 6) { // full binary tree
    for (int i = 1; i < n; ++i) {
      ensuref(parents[i] + 1 == (i + 1) / 2, "tree is not full binary tree"); 
    }
  }

  if (group == 7) { // h <= 100
    const int kMaxH = 100; 
    vector<int> depth(n);
    for (int i = 1; i < n; ++i) {
      depth[i] = depth[parents[i]] + 1;
      ensuref(depth[i] <= kMaxH, "depth exeed %d", kMaxH);
    }
  }

  if (group == 8) { // min heap
    for (int i = 1; i < n; ++i) {
      ensuref(skills[parents[i]] < skills[i], "skills are not heap-like");
    }
  }
}
