#include <iostream>
#include <vector>
#include <algorithm>
#include <queue>

using namespace std;

int main() {
  ios_base::sync_with_stdio(false); cin.tie(0);
  int n, q;
  cin >> n >> q;
  vector<int> parents(n, -1); 
  for (int i = 1; i < n; ++i) {
    cin >> parents[i];
    --parents[i];
  }
  vector<int> skills(n);
  for (int i = 0; i < n; ++i) {
    cin >> skills[i];
    --skills[i];
  }
  vector<int> queries(q); 
  for (int i = 0; i < q; ++i) {
    cin >> queries[i];
    --queries[i];
  }

  vector<vector<int>> graph(n); 
  for (int i = 1; i < n; ++i) {
    graph[parents[i]].push_back(i);
  }

  auto simulate_full_order = [n, &graph](const vector<int> &skills) {
    priority_queue<pair<int, int>> pq;
    pq.emplace(skills[0], 0);
    vector<int> positions(n); 
    vector<int> order;
    order.reserve(n);
    int ptr = 0; 
    while (!pq.empty()) {
      int u = pq.top().second;
      pq.pop();
      positions[u] = ptr++;
      order.push_back(u);
      for (auto v : graph[u]) {
        pq.emplace(skills[v], v);
      }
    }
    return make_pair(order, positions);
  };

  auto [order, positions] = simulate_full_order(skills); 
  
  vector<int> bamboo_idx(n, -1);
  int bamboo_ptr = -1; 
  for (int i = 1; i < n; ++i) {
    if (parents[i] == 0) {
      ++bamboo_ptr; 
    }
    bamboo_idx[i] = bamboo_ptr;
  }

  int bamboos_num = bamboo_ptr + 1;
  vector<int> count_in_bamboo(bamboos_num);

  vector<int> qr_ids(n, -1);
  for (int qr_id = 0; qr_id < q; ++qr_id) {
    qr_ids[queries[qr_id]] = qr_id; 
  }

  vector<int> answers(q, n);
  
  int best_bamboo = -1; 
  int second_best_bamboo = -1; 

  auto update_bests_with_idx = [&best_bamboo, &second_best_bamboo, &count_in_bamboo](int bamboo) {
    if (best_bamboo == bamboo) {
    } else if (second_best_bamboo == bamboo) {
      if (count_in_bamboo[second_best_bamboo] > count_in_bamboo[best_bamboo]) {
        swap(best_bamboo, second_best_bamboo); 
      }
    } else {
      if (best_bamboo == -1 || count_in_bamboo[bamboo] > count_in_bamboo[best_bamboo]) {
        swap(bamboo, best_bamboo);
      }
      if (bamboo != -1 && (second_best_bamboo == -1 || count_in_bamboo[bamboo] > count_in_bamboo[second_best_bamboo])) {
        swap(bamboo, second_best_bamboo);
      }
    }
  };

  auto get_max_except_idx = [&count_in_bamboo, &best_bamboo, &second_best_bamboo] (int bamboo) {
    if (bamboo != best_bamboo) {
      return (best_bamboo == -1 ? 0 : count_in_bamboo[best_bamboo]); 
    } else {
      return (second_best_bamboo == -1 ? 0 : count_in_bamboo[second_best_bamboo]); 
    }
  }; 
  
  for (int ord_pos = 0; ord_pos < n; ++ord_pos) {
    int u = order[ord_pos];
    if (qr_ids[u] != -1) {
      answers[qr_ids[u]] = positions[u] - (u == 0 ? 0 : get_max_except_idx(bamboo_idx[u]));
    }
    if (u != 0) {
      ++count_in_bamboo[bamboo_idx[u]]; 
      update_bests_with_idx(bamboo_idx[u]);
    }
  }

  for (int qr_ids = 0; qr_ids < q; ++qr_ids) {
    cout << answers[qr_ids] << ' ';
  }
}
