#include <cstdint>
#include <iostream>
#include <cmath>
#include <cctype>
#include <string>
#include <vector>
#include <algorithm>
#include <set>
#include <map>
#include <deque>
#include <stack>
#include <unordered_set>
#include <sstream>
#include <cstring>
#include <iomanip>
#include <queue>
#include <unordered_map>
#include <random>
#include <cfloat>
#include <chrono>
#include <bitset>
#include <complex>
#include <functional>
#include <immintrin.h>
#include <memory>

void dfs(int32_t v, std::vector<int32_t>* children, int32_t* subtree_size) {
    subtree_size[v] = 1;
    for(int32_t i = 0; i < children[v].size(); i++) {
        dfs(children[v][i], children, subtree_size);
        subtree_size[v] += subtree_size[children[v][i]];
    }
}

void dfs_reorder(int32_t v, std::vector<int32_t>* children, int32_t heavy_path_ind, int32_t& next_heavy_path, int32_t* reordered_heavy_path_ind,
                 int32_t* reordered_heavy_path_top, int32_t& reordered_ind, std::vector<int32_t>& new_to_old, int32_t* subtree_size) {
    reordered_heavy_path_ind[reordered_ind] = heavy_path_ind;
    if(reordered_heavy_path_top[heavy_path_ind] == -1)
        reordered_heavy_path_top[heavy_path_ind] = reordered_ind;
    reordered_ind++;
    new_to_old.push_back(v);
    if(children[v].empty())
        return;
    
    for(int32_t i = 0; i < children[v].size(); i++)
        if(subtree_size[children[v][i]] > subtree_size[children[v][0]])
            std::swap(children[v][0], children[v][i]);

    dfs_reorder(children[v][0], children, heavy_path_ind, next_heavy_path, reordered_heavy_path_ind, reordered_heavy_path_top, reordered_ind, new_to_old, subtree_size);
    for(int32_t i = 1; i < children[v].size(); i++) {
        int32_t new_heavy_path_ind = next_heavy_path++;
        dfs_reorder(children[v][i], children, new_heavy_path_ind, next_heavy_path, reordered_heavy_path_ind, reordered_heavy_path_top, reordered_ind, new_to_old, subtree_size);
    }
}

int64_t* segtree;
int64_t* lazy;

void propagate(int32_t node) {
    segtree[node * 2 + 1] += lazy[node];
    segtree[node * 2 + 2] += lazy[node];
    lazy[node * 2 + 1] += lazy[node];
    lazy[node * 2 + 2] += lazy[node];
    lazy[node] = 0;
}

void add(int32_t node, int32_t left, int32_t right, int32_t query_left, int32_t query_right, int32_t x) {
    if(left >= query_right || right <= query_left)
        return;
    if(left >= query_left && right <= query_right) {
        lazy[node] += x;
        segtree[node] += x;
        return;
    }
    int32_t m = (left + right) / 2;
    propagate(node);
    add(node * 2 + 1, left, m, query_left, query_right, x);
    add(node * 2 + 2, m, right, query_left, query_right, x);
    segtree[node] = std::max(segtree[node * 2 + 1], segtree[node * 2 + 2]);
}

int main() {
    std::ios::sync_with_stdio(false);
    std::cin.tie(0);
    
    int32_t n, num_q;
    std::cin >> n >> num_q;

    std::vector<int32_t>* children = new std::vector<int32_t>[n];
    int32_t* parents = new int32_t[n];
    parents[0] = -1;
    for(int32_t i = 0; i < n - 1; i++) {
        int32_t parent;
        std::cin >> parent;
        parent--;
        parents[i + 1] = parent;

        children[parent].push_back(i + 1);
    }

    int32_t* arr = new int32_t[n];
    for(int32_t i = 0; i < n; i++)
        std::cin >> arr[i];

    int32_t* subtree_size = new int32_t[n];
    dfs(0, children, subtree_size);

    int32_t next_heavy_path = 1;
    int32_t* reordered_heavy_path_ind = new int32_t[n];
    int32_t* reordered_heavy_path_top = new int32_t[n];
    for(int32_t i = 0; i < n; i++)
        reordered_heavy_path_top[i] = -1;
    int32_t reordered_ind = 0;
    std::vector<int32_t> new_to_old;
    dfs_reorder(0, children, 0, next_heavy_path, reordered_heavy_path_ind, reordered_heavy_path_top, reordered_ind, new_to_old, subtree_size);

    int32_t* old_to_new = new int32_t[n];
    for(int32_t i = 0; i < n; i++)
        old_to_new[new_to_old[i]] = i;
    
    std::vector<int32_t>* children2 = new std::vector<int32_t>[n];
    for(int32_t i = 0; i < n; i++)
        for(int32_t v : children[i])
            children2[old_to_new[i]].push_back(old_to_new[v]);
    int32_t* parents2 = new int32_t[n];
    parents2[0] = -1;
    for(int32_t i = 1; i < n; i++)
        parents2[old_to_new[i]] = old_to_new[parents[i]];
    children = children2;
    parents = parents2;

    // Segtree
    segtree = new int64_t[4 * n];
    lazy = new int64_t[4 * n];
    for(int32_t i = 0; i < 4 * n; i++) {
        segtree[i] = 0;
        lazy[i] = 0;
    }

    // Actual solution
    int32_t* ans = new int32_t[n];
    ans[0] = 0;
    std::set<std::pair<int32_t, int32_t>> active;
    for(int32_t i = 0; i < children[0].size(); i++)
        active.emplace(arr[new_to_old[children[0][i]]], children[0][i]);
    /*int32_t* score = new int32_t[n];
    for(int32_t i = 0; i < n; i++)
        score[i] = 0;*/
    int32_t timer_ = 1;
    while(!active.empty()) {
        auto [dummy, v] = *active.rbegin();
        auto it = active.end();
        it--;
        active.erase(it);

        int32_t cur = v;
        /*while(cur != -1) {
            score[cur] -= 1'000'000;
            cur = parents[cur];
        }*/
        while(cur != -1) {
            int32_t top_cur = reordered_heavy_path_top[reordered_heavy_path_ind[cur]];
            add(0, 0, n, top_cur, cur + 1, -1'000'000);
            cur = parents[top_cur];
        }

        int32_t boost = 0;
        /*for(int32_t i = 0; i < n; i++)
            boost = std::max(boost, score[i]);*/
        boost = segtree[0];

        cur = v;
        /*while(cur != -1) {
            score[cur] += 1'000'001;
            cur = parents[cur];
        }*/
        while(cur != -1) {
            int32_t top_cur = reordered_heavy_path_top[reordered_heavy_path_ind[cur]];
            add(0, 0, n, top_cur, cur + 1, 1'000'001);
            cur = parents[top_cur];
        }

        ans[v] = timer_ - boost;
        timer_++;
        for(int32_t i = 0; i < children[v].size(); i++)
            active.emplace(arr[new_to_old[children[v][i]]], children[v][i]);
    }

    for(int32_t q = 0; q < num_q; q++) {
        int32_t v;
        std::cin >> v;
        v--;

        std::cout << ans[old_to_new[v]] << " ";
    }
    std::cout << "\n";
}





