#include <iostream>
#include <vector>
#include <algorithm>
#include <queue>
#include <cassert>

using namespace std;

int main() {
  ios_base::sync_with_stdio(false); cin.tie(0);
  int n, q;
  cin >> n >> q;
  vector<int> parents(n, -1); 
  for (int i = 1; i < n; ++i) {
    cin >> parents[i];
    --parents[i];
  }
  vector<int> skills(n);
  for (int i = 0; i < n; ++i) {
    cin >> skills[i];
    --skills[i];
  }
  vector<int> queries(q); 
  for (int i = 0; i < q; ++i) {
    cin >> queries[i];
    --queries[i];
  }

  vector<vector<int>> graph(n); 
  for (int i = 1; i < n; ++i) {
    graph[parents[i]].push_back(i);
  }

  auto simulate_full_order = [n, &graph](const vector<int> &skills) {
    priority_queue<pair<int, int>> pq;
    pq.emplace(skills[0], 0);
    vector<int> positions(n); 
    vector<int> order;
    order.reserve(n);
    int ptr = 0; 
    while (!pq.empty()) {
      int u = pq.top().second;
      pq.pop();
      positions[u] = ptr++;
      order.push_back(u);
      for (auto v : graph[u]) {
        pq.emplace(skills[v], v);
      }
    }
    return make_pair(order, positions);
  };

  auto [order, positions] = simulate_full_order(skills); 
  
  vector<int> on(n);
  vector<int> subtr_sums(n);

  vector<int> qr_ids(n, -1);
  for (int qr_id = 0; qr_id < q; ++qr_id) {
    qr_ids[queries[qr_id]] = qr_id; 
  }

  vector<int> answers(q, n);
  vector<pair<int, int>> max_child(n, {0, -1});
  vector<pair<int, int>> second_max_child(n, {0, -1}); 

  auto update_two_max_children_with = [&max_child, &second_max_child, &parents] (int u, int sum) {
    int par = parents[u];
    assert(par != -1);

    if (max_child[par].second == u) {
      max_child[par].first = sum;
    } else if (second_max_child[par].second == u) {
      second_max_child[par].first = sum; 
      if (second_max_child[par] > max_child[par]) {
        swap(max_child[par], second_max_child[par]);
      }
    } else {
      auto new_child = make_pair(sum, u); 
      if (new_child > max_child[par]) {
        swap(new_child, max_child[par]);
      }
      if (new_child > second_max_child[par]) {
        swap(new_child, second_max_child[par]);
      }
    }
  }; 

  for (int ord_pos = 0; ord_pos < n; ++ord_pos) {
    int u = order[ord_pos];
    int v = u; 
    while (v != -1) {
      subtr_sums[v] += 1;
      if (parents[v] != -1) {
        update_two_max_children_with(v, subtr_sums[v]);
      }
      v = parents[v];
    }
    if (qr_ids[u] != -1) {
      int& qr_ans = answers[qr_ids[u]];
      int init_pos = positions[u];
      qr_ans = init_pos;
      int v = parents[u];
      int child = u;
      while (v != -1) {
        if (max_child[v].second != child) {
          qr_ans = min(qr_ans, init_pos - max_child[v].first);
        } else {
          qr_ans = min(qr_ans, init_pos - second_max_child[v].first);
        }
        child = v; 
        v = parents[v];
      }
    }
    on[u] = 1;
  }

  for (int qr_ids = 0; qr_ids < q; ++qr_ids) {
    cout << answers[qr_ids] << ' ';
  }
}
