#include <bits/stdc++.h>

using namespace std;


class segtree {
 public:
  struct node {
    // don't forget to set default value (used for leaves)
    // not necessarily neutral element!
    int max_value = 0;

    void apply(int l, int r, int v) {
      max_value = max(max_value, v);
    }
  };

  node unite(const node &a, const node &b) const {
    node res;
    res.max_value = max(a.max_value, b.max_value);
    return res;
  }

  inline void push(int x, int l, int r) {
    int y = (l + r) >> 1;
    int z = x + ((y - l + 1) << 1);
    // push from x into (x + 1) and z
/*
    if (tree[x].add != 0) {
      tree[x + 1].apply(l, y, tree[x].add);
      tree[z].apply(y + 1, r, tree[x].add);
      tree[x].add = 0;
    }
*/
  }

  inline void pull(int x, int z) {
    tree[x] = unite(tree[x + 1], tree[z]);
  }

  int n;
  vector<node> tree;

  void build(int x, int l, int r) {
    if (l == r) {
      return;
    }
    int y = (l + r) >> 1;
    int z = x + ((y - l + 1) << 1);
    build(x + 1, l, y);
    build(z, y + 1, r);
    pull(x, z);
  }

  template <typename M>
  void build(int x, int l, int r, const vector<M> &v) {
    if (l == r) {
      tree[x].apply(l, r, v[l]);
      return;
    }
    int y = (l + r) >> 1;
    int z = x + ((y - l + 1) << 1);
    build(x + 1, l, y, v);
    build(z, y + 1, r, v);
    pull(x, z);
  }

  node get(int x, int l, int r, int ll, int rr) {
    if (ll <= l && r <= rr) {
      return tree[x];
    }
    int y = (l + r) >> 1;
    int z = x + ((y - l + 1) << 1);
    push(x, l, r);
    node res{};
    if (rr <= y) {
      res = get(x + 1, l, y, ll, rr);
    } else {
      if (ll > y) {
        res = get(z, y + 1, r, ll, rr);
      } else {
        res = unite(get(x + 1, l, y, ll, rr), get(z, y + 1, r, ll, rr));
      }
    }
    pull(x, z);
    return res;
  }

  template <typename... M>
  void modify(int x, int l, int r, int ll, int rr, const M&... v) {
    if (ll <= l && r <= rr) {
      tree[x].apply(l, r, v...);
      return;
    }
    int y = (l + r) >> 1;
    int z = x + ((y - l + 1) << 1);
    push(x, l, r);
    if (ll <= y) {
      modify(x + 1, l, y, ll, rr, v...);
    }
    if (rr > y) {
      modify(z, y + 1, r, ll, rr, v...);
    }
    pull(x, z);
  }

  int find_first_knowingly(int x, int l, int r, const function<bool(const node&)> &f) {
    if (l == r) {
      return l;
    }
    push(x, l, r);
    int y = (l + r) >> 1;
    int z = x + ((y - l + 1) << 1);
    int res;
    if (f(tree[x + 1])) {
      res = find_first_knowingly(x + 1, l, y, f);
    } else {
      res = find_first_knowingly(z, y + 1, r, f);
    }
    pull(x, z);
    return res;
  }

  int find_first(int x, int l, int r, int ll, int rr, const function<bool(const node&)> &f) {
    if (ll <= l && r <= rr) {
      if (!f(tree[x])) {
        return -1;
      }
      return find_first_knowingly(x, l, r, f);
    }
    push(x, l, r);
    int y = (l + r) >> 1;
    int z = x + ((y - l + 1) << 1);
    int res = -1;
    if (ll <= y) {
      res = find_first(x + 1, l, y, ll, rr, f);
    }
    if (rr > y && res == -1) {
      res = find_first(z, y + 1, r, ll, rr, f);
    }
    pull(x, z);
    return res;
  }

  int find_last_knowingly(int x, int l, int r, const function<bool(const node&)> &f) {
    if (l == r) {
      return l;
    }
    push(x, l, r);
    int y = (l + r) >> 1;
    int z = x + ((y - l + 1) << 1);
    int res;
    if (f(tree[z])) {
      res = find_last_knowingly(z, y + 1, r, f);
    } else {
      res = find_last_knowingly(x + 1, l, y, f);
    }
    pull(x, z);
    return res;
  }

  int find_last(int x, int l, int r, int ll, int rr, const function<bool(const node&)> &f) {
    if (ll <= l && r <= rr) {
      if (!f(tree[x])) {
        return -1;
      }
      return find_last_knowingly(x, l, r, f);
    }
    push(x, l, r);
    int y = (l + r) >> 1;
    int z = x + ((y - l + 1) << 1);
    int res = -1;
    if (rr > y) {
      res = find_last(z, y + 1, r, ll, rr, f);
    }
    if (ll <= y && res == -1) {
      res = find_last(x + 1, l, y, ll, rr, f);
    }
    pull(x, z);
    return res;
  }

  segtree(int _n) : n(_n) {
    assert(n > 0);
    tree.resize(2 * n - 1);
    build(0, 0, n - 1);
  }

  template <typename M>
  segtree(const vector<M> &v) {
    n = v.size();
    assert(n > 0);
    tree.resize(2 * n - 1);
    build(0, 0, n - 1, v);
  }

  node get(int ll, int rr) {
    assert(0 <= ll && ll <= rr && rr <= n - 1);
    return get(0, 0, n - 1, ll, rr);
  }

  node get(int p) {
    assert(0 <= p && p <= n - 1);
    return get(0, 0, n - 1, p, p);
  }

  template <typename... M>
  void modify(int ll, int rr, const M&... v) {
    assert(0 <= ll && ll <= rr && rr <= n - 1);
    modify(0, 0, n - 1, ll, rr, v...);
  }

  // find_first and find_last call all FALSE elements
  // to the left (right) of the sought position exactly once

  int find_first(int ll, int rr, const function<bool(const node&)> &f) {
    assert(0 <= ll && ll <= rr && rr <= n - 1);
    return find_first(0, 0, n - 1, ll, rr, f);
  }

  int find_last(int ll, int rr, const function<bool(const node&)> &f) {
    assert(0 <= ll && ll <= rr && rr <= n - 1);
    return find_last(0, 0, n - 1, ll, rr, f);
  }
};

template <typename T>
class FenwickTree {
 public:
  vector<T> fenw;
  int n;
  int pw;

  FenwickTree() : n(0) {}
  FenwickTree(int n_) : n(n_) {
    fenw.resize(n);
    pw = bit_floor(unsigned(n));
  }

  void Modify(int x, T v) {
    assert(0 <= x && x <= n);
    while (x < n) {
      fenw[x] += v;
      x |= x + 1;
    }
  }

  T Query(int x) {
    assert(0 <= x && x <= n);
    T v{};
    while (x > 0) {
      v += fenw[x - 1];
      x &= x - 1;
    }
    return v;
  }

  // Returns the length of the longest prefix with sum <= c
  int MaxPrefix(T c) {
    T v{};
    int at = 0;
    for (int len = pw; len > 0; len >>= 1) {
      if (at + len <= n) {
        auto nv = v;
        nv += fenw[at + len - 1];
        if (!(c < nv)) {
          v = nv;
          at += len;
        }
      }
    }
    assert(0 <= at && at <= n);
    return at;
  }
}; 

class HeavyLightDecomposition {
public:
  HeavyLightDecomposition(const vector<vector<int>> &graph, const vector<int>& parents) : 
    n(static_cast<int>(graph.size())), 
    graph(graph), 
    parents(parents), 
    subtree_sums(FenwickTree<int>(n)), 
    path_maxs(segtree(n)) 
  {
    subtree_sizes.resize(n);
    heads.resize(n);
    entry_times.resize(n);
    exit_times.resize(n);

    max_light_child.resize(n, {0, -1});
    second_max_light_child.resize(n, {0, -1}); 

    reorder_children(0);
    decompose(0, 0);

  }

  void switch_on(int u) {      
    subtree_sums.Modify(entry_times[u], 1);

    u = heads[u];
    while (parents[u] != -1) {
      int sum = get_subtree_sum(u);
      update_two_max_children_with(u, sum);
      
      int par = parents[u];
      path_maxs.modify(entry_times[par], entry_times[par], sum);

      u = heads[par];
    }
  }

  int max_adjacent_subtree_sum(int u) {
    int result = 0;
    while (u != -1) {
      int head = heads[u]; 
      if (u != head) {
        result = max(result, path_maxs.get(entry_times[head], entry_times[parents[u]]).max_value); 
      }
      u = head; 

      int par = parents[u];
      if (par != -1) {
        if (u != max_light_child[par].second) {
          result = max(result, max_light_child[par].first);
        } else {
          result = max(result, second_max_light_child[par].first);
        }

        int heavy_child = graph[par][0];
        result = max(result, get_subtree_sum(heavy_child));
      }
      u = par;
    }
    return result; 
  }

private:
  int get_subtree_sum(int u) {
    return subtree_sums.Query(exit_times[u]) - subtree_sums.Query(entry_times[u]);
  }

  void update_two_max_children_with(int u, int sum) {
    int par = parents[u];
    assert(par != -1);

    if (max_light_child[par].second == u) {
      max_light_child[par].first = sum;
    } else if (second_max_light_child[par].second == u) {
      second_max_light_child[par].first = sum; 
      if (second_max_light_child[par] > max_light_child[par]) {
        swap(max_light_child[par], second_max_light_child[par]);
      }
    } else {
      auto new_child = make_pair(sum, u); 
      if (new_child > max_light_child[par]) {
        swap(new_child, max_light_child[par]);
      }
      if (new_child > second_max_light_child[par]) {
        swap(new_child, second_max_light_child[par]);
      }
    }
  }

  void reorder_children(int u) {
    subtree_sizes[u] = 1;
    for (auto v : graph[u]) {
      reorder_children(v);
      subtree_sizes[u] += subtree_sizes[v];
    }
    nth_element(graph[u].begin(), graph[u].begin(), graph[u].end(), [&](int a, int b) {
      return subtree_sizes[a] > subtree_sizes[b];
    });
  }

  void decompose(int u, int chain_head) {
    entry_times[u] = hld_order_ptr++;
    heads[u] = chain_head;
    if (!graph[u].empty()) {
      decompose(graph[u][0], chain_head);
      for (int i = 1; i < static_cast<int>(graph[u].size()); ++i) {
        decompose(graph[u][i], graph[u][i]);
      }
    }
    exit_times[u] = hld_order_ptr;
  }

private:
  int n;
  vector<vector<int>> graph; 
  vector<int> parents; 
  
  int hld_order_ptr = 0;
  vector<int> subtree_sizes; 
  vector<int> heads; 
  vector<int> entry_times; 
  vector<int> exit_times;

  vector<pair<int, int>> max_light_child; 
  vector<pair<int, int>> second_max_light_child; 

  FenwickTree<int> subtree_sums; 
  segtree path_maxs;
};

int main() {
  ios_base::sync_with_stdio(false); cin.tie(0);
  int n, q;
  cin >> n >> q;
  vector<int> parents(n, -1); 
  for (int i = 1; i < n; ++i) {
    cin >> parents[i];
    --parents[i];
  }
  vector<int> skills(n);
  for (int i = 0; i < n; ++i) {
    cin >> skills[i];
    --skills[i];
  }
  vector<int> queries(q); 
  for (int i = 0; i < q; ++i) {
    cin >> queries[i];
    --queries[i];
  }

  vector<vector<int>> graph(n); 
  for (int i = 1; i < n; ++i) {
    graph[parents[i]].push_back(i);
  }

  HeavyLightDecomposition hld(graph, parents);

  auto simulate_full_order = [n, &graph](const vector<int> &skills) {
    priority_queue<pair<int, int>> pq;
    pq.emplace(skills[0], 0);
    vector<int> positions(n); 
    vector<int> order;
    order.reserve(n);
    int ptr = 0; 
    while (!pq.empty()) {
      int u = pq.top().second;
      pq.pop();
      positions[u] = ptr++;
      order.push_back(u);
      for (auto v : graph[u]) {
        pq.emplace(skills[v], v);
      }
    }
    return make_pair(order, positions);
  };

  auto [order, positions] = simulate_full_order(skills); 
  
  vector<int> qr_ids(n, -1);
  for (int qr_id = 0; qr_id < q; ++qr_id) {
    qr_ids[queries[qr_id]] = qr_id; 
  }

  vector<int> answers(q, n);

  for (int ord_pos = 0; ord_pos < n; ++ord_pos) {
    int u = order[ord_pos];
    if (qr_ids[u] != -1) {
      answers[qr_ids[u]] = positions[u] - hld.max_adjacent_subtree_sum(u);
    }
    hld.switch_on(u);
  }

  for (int qr_ids = 0; qr_ids < q; ++qr_ids) {
    cout << answers[qr_ids] << ' ';
  }
}
