#include <iostream>
#include <vector>
#include <algorithm>
#include <queue>

using namespace std;

int main() {
  ios_base::sync_with_stdio(false); cin.tie(0);
  int n, q;
  cin >> n >> q;
  vector<int> parents(n, -1); 
  for (int i = 1; i < n; ++i) {
    cin >> parents[i];
    --parents[i];
  }
  vector<int> skills(n);
  for (int i = 0; i < n; ++i) {
    cin >> skills[i];
    --skills[i];
  }
  vector<int> queries(q); 
  for (int i = 0; i < q; ++i) {
    cin >> queries[i];
    --queries[i];
  }

  vector<vector<int>> graph(n); 
  for (int i = 1; i < n; ++i) {
    graph[parents[i]].push_back(i);
  }

  auto simulate_full_order = [n, &graph](const vector<int> &skills) {
    priority_queue<pair<int, int>> pq;
    pq.emplace(skills[0], 0);
    vector<int> positions(n); 
    vector<int> order;
    order.reserve(n);
    int ptr = 0; 
    while (!pq.empty()) {
      int u = pq.top().second;
      pq.pop();
      positions[u] = ptr++;
      order.push_back(u);
      for (auto v : graph[u]) {
        pq.emplace(skills[v], v);
      }
    }
    return make_pair(order, positions);
  };

  auto [order, positions] = simulate_full_order(skills); 
  
  vector<int> on(n);
  vector<int> subtr_sums(n);

  vector<int> qr_ids(n, -1);
  for (int qr_id = 0; qr_id < q; ++qr_id) {
    qr_ids[queries[qr_id]] = qr_id; 
  }

  vector<int> answers(q, n);

  for (int ord_pos = 0; ord_pos < n; ++ord_pos) {
    int u = order[ord_pos];
    if (qr_ids[u] != -1) {
      for (int i = n - 1; i >= 0; --i) {
        subtr_sums[i] = on[i];
        for (int j : graph[i]) {
          subtr_sums[i] += subtr_sums[j];
        }
      }
      int& qr_ans = answers[qr_ids[u]];
      int init_pos = positions[u];
      qr_ans = init_pos;
      int v = parents[u];
      int child = u;
      while (v != -1) {
        for (int change : graph[v]) {
          if (change != child) {
            qr_ans = min(qr_ans, init_pos - subtr_sums[change]);
          }
        }
        child = v; 
        v = parents[v];
      }
    }
    on[u] = 1;
  }

  for (int qr_ids = 0; qr_ids < q; ++qr_ids) {
    cout << answers[qr_ids] << ' ';
  }
}
